unit Sample10;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim, ComCtrls;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    ProgressBar: TProgressBar;
    Entrance: TGenerator;
    Hall: TQueue;
    Barbers: TStorage;
    ExitDoor: TTerminator;
    AnotherShop: TTerminator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure EntranceExit(Sender: TBlock; Trans: TTransaction);
    procedure EntranceBeforeTimeGoOn(Sender: TAggregate);
    procedure EntranceRouting(Sender: TBlock; Trans: TTransaction);
    procedure HallEnter(Sender: TBlock; Trans: TTransaction);
    procedure HallRouting(Sender: TBlock; Trans: TTransaction);
    procedure HallTimeFinish(Sender: TBlock; Trans: TTransaction);
    procedure BarbersEnter(Sender: TBlock; Trans: TTransaction);
    procedure BarbersRouting(Sender: TBlock; Trans: TTransaction);
    procedure ModelNewTime(Sender: TAggregate; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;



var
  Form1: TForm1;
  LimitTime: real;        // Simulation time
  QCapacity: longint;     // Queue capacity

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
var i: integer;
begin
   LimitTime:=720.0;
   Memo.Lines.Clear;
   Memo.Lines.Add('Barbers     Losses        Usage of one barber');
   for i:=1 to 8 do
   begin
      Barbers.Capacity:=i;
      Model.Simulate(LimitTime);
      Memo.Lines.Add(IntToStr(Barbers.Capacity)+'           '+FormatFloat('00.00',
                     AnotherShop.Entries*100.0/Entrance.Exits)+'%        '+
                     FormatFloat('00.00',Barbers.AverageCount*100.0/Barbers.Capacity)+'%');
      Model.Reset;
   end;
end;


procedure TForm1.EntranceBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.EntranceExit(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(3.0));
end;

procedure TForm1.EntranceRouting(Sender: TBlock; Trans: TTransaction);
begin
  if Hall.isReadyToReceive(Trans) then
     Sender.PassTo(Hall)
  else
     Sender.PassTo(AnotherShop);
end;

procedure TForm1.HallEnter(Sender: TBlock; Trans: TTransaction);
begin
  if MultiRand.Uniform(0.0,1.0) > 0.5 then
     Sender.NextTime(MultiRand.Uniform(20.0,30.0));
end;

procedure TForm1.HallRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Barbers);
end;

procedure TForm1.HallTimeFinish(Sender: TBlock; Trans: TTransaction);
begin
   Sender.PassTo(AnotherShop);          // Pass to terminator
end;

procedure TForm1.BarbersEnter(Sender: TBlock; Trans: TTransaction);
begin
   Sender.NextTime(MultiRand.Exponential(10.0));
end;

procedure TForm1.BarbersRouting(Sender: TBlock; Trans: TTransaction);
begin
   Sender.PassTo(ExitDoor);
end;

procedure TForm1.ModelNewTime(Sender: TAggregate; Trans: TTransaction);
var  NewPos: integer;
     Ratio: real;
begin
   Ratio:=ModelTime/LimitTime;
   if Ratio>1.0 then Ratio:=1.0;
   NewPos:=Round(Ratio*100.0);
   with ProgressBar do
      if Position<>NewPos then Position:=NewPos;
end;

end.
